@@ -33,7 +33,15 @@ $cmd = "svn export $repo qgvdial-$qver";
 system($cmd);
 
 # Version replacement
-$cmd = "perl $basedir/build-files/version.pl __QGVDIAL_VERSION__ $qver $basedir";
+$cmd = "perl version.pl __QGVDIAL_VERSION__ $qver $basedir";
+print "$cmd\n";
+system($cmd);
+
+# Cipher replacement
+open my $qgvcipfile, '<', "cipher_qgvdial";
+my $cipher = <$qgvcipfile>;
+close $qgvcipfile;
+$cmd = "perl version.pl __THIS_IS_MY_EXTREMELY_LONG_KEY_ '$cipher' $basedir";
 print "$cmd\n";
 system($cmd);
 
