@@ -54,6 +54,14 @@ $cmd = "perl version.pl __CHANGELOG_DATETIME__ '$dtstr' $basedir";
 print "$cmd\n";
 system($cmd);
 
+# Cipher replacement
+open my $qgvcipfile, '<', "cipher_qgvdial";
+my $cipher = <$qgvcipfile>;
+close $qgvcipfile;
+$cmd = "perl version.pl __THIS_IS_MY_EXTREMELY_LONG_KEY_ '$cipher' $basedir";
+print "$cmd\n";
+system($cmd);
+
 # Do everything upto the preparation of the debian directory. Code is still not compiled.
 $cmd = "cd $basedir/qgvdial ; mv harmattan qgvdial-$qver";
 print "$cmd\n";
