@@ -52,11 +52,19 @@ open(PRO_FILE, ">>qgvdial-$qver/qgvdial/symbian/symbian.pro") || die "Cannot ope
 print PRO_FILE "VERSION=__QGVDIAL_VERSION__\n";
 close PRO_FILE;
 
-# Version changes
-$cmd = "cd qgvdial-$qver & perl ./build-files/version.pl __QGVDIAL_VERSION__ $qver";
+# Version replacement
+$cmd = "perl version.pl __QGVDIAL_VERSION__ $qver qgvdial-$qver";
 print("$cmd\n");
 system($cmd);
 
+# Cipher replacement
+open my $qgvcipfile, '<', "../cipher_qgvdial";
+my $cipher = <$qgvcipfile>;
+close $qgvcipfile;
+$cmd = "perl version.pl __THIS_IS_MY_EXTREMELY_LONG_KEY_ '$cipher' qgvdial-$qver";
+print "$cmd\n";
+#system($cmd);
+
 # Mosquitto is merged straight into the build
 $cmd = "cd qgvdial-$qver/qgvdial/symbian & prep_s1.bat";
 print "$cmd\n";
