@@ -39,6 +39,14 @@ close PRO_FILE;
 # Version replacement
 system("cd qgvdial-$qver & perl build-files/version.pl __QGVDIAL_VERSION__ $qver");
 
+# Cipher replacement
+open my $qgvcipfile, '<', "../qgvdial_cipher_key";
+my $cipher = <$qgvcipfile>;
+close $qgvcipfile;
+$cmd = "perl version.pl __THIS_IS_MY_EXTREMELY_LONG_KEY_ '$cipher' qgvdial-$qver";
+print "$cmd\n";
+#system($cmd);
+
 # Replace the QTDIR variable
 $cmd = `echo %QTDIR%`;
 $cmd =~ s/\\/\\\\/g;
