@@ -35,8 +35,7 @@ MainWindow::MainWindow (QWidget *parent)
 , m_registeredPhonesModel(NULL)
 , oContacts (NULL)
 , oInbox (NULL)
-, bBeginPlayAfterLoad (false)
-, vmailPlayer (NULL)
+, oVmail (NULL)
 , nwMgr (NULL)
 , statusTimer (this)
 #ifdef Q_WS_MAEMO_5
@@ -65,6 +64,8 @@ MainWindow::MainWindow (QWidget *parent)
 , contactsTimer (this)
 , inboxTimer (this)
 {
+    bool rv;
+
     modelRegNumber = new RegNumberModel(this);
     if (modelRegNumber == NULL) {
         Q_WARN("Failed to allocate modelRegNumber");
@@ -77,13 +78,24 @@ MainWindow::MainWindow (QWidget *parent)
         exit(1);
         return;
     }
-    oInbox = new GVInbox(gvApi, this);
+
+    oInbox = new GVInbox(this);
     if (oInbox == NULL) {
         Q_WARN("Failed to allocate oInbox");
         exit(1);
         return;
     }
 
+    oVmail = new qgvVmail(this);
+    if (oVmail == NULL) {
+        Q_WARN("Failed to allocate oVmail");
+        exit(1);
+        return;
+    }
+    rv = connect(oVmail, SIGNAL(setStatus(QString,int)),
+                 this, SLOT(setStatus(QString,int)));
+    Q_ASSERT(rv);
+
     initLogging ();
 
     qRegisterMetaType<ContactInfo>("ContactInfo");
@@ -93,7 +105,6 @@ MainWindow::MainWindow (QWidget *parent)
 
     initQML ();
 
-    bool rv;
     // A systray icon if the OS supports it
     if (QSystemTrayIcon::isSystemTrayAvailable ())
     {
@@ -141,9 +152,9 @@ MainWindow::~MainWindow ()
     mqThread.terminate ();
 #endif
 
-    if (NULL != vmailPlayer) {
-        delete vmailPlayer;
-        vmailPlayer = NULL;
+    if (NULL != oVmail) {
+        delete oVmail;
+        oVmail = NULL;
     }
 
     CacheDatabase &dbMain = Singletons::getRef().getDBMain ();
@@ -345,8 +356,8 @@ MainWindow::init ()
     if (!rv) { exit(1); }
 
     // Status from contacts object
-    rv = connect (oContacts, SIGNAL (status   (const QString &, int)),
-                  this     , SLOT   (setStatus(const QString &, int)));
+    rv = connect (oContacts, SIGNAL(status(const QString&,int)),
+                  this     , SLOT(setStatus(const QString&,int)));
     Q_ASSERT(rv);
     if (!rv) { exit(1); }
     // oContacts->allContacts -> this.getContactsDone
@@ -355,8 +366,8 @@ MainWindow::init ()
     Q_ASSERT(rv);
     if (!rv) { exit(1); }
     // Status from inbox object
-    rv = connect (oInbox, SIGNAL (status   (const QString &, int)),
-                  this  , SLOT   (setStatus(const QString &, int)));
+    rv = connect (oInbox, SIGNAL(status(const QString&,int)),
+                  this  , SLOT(setStatus(const QString&,int)));
     Q_ASSERT(rv);
     if (!rv) { exit(1); }
 
@@ -466,7 +477,7 @@ MainWindow::init ()
         this->setUsername (strUser);
         this->setPassword (strPass);
 
-        logoutCompleted (NULL);
+        logoutWork (NULL);
         // Login without popping up the "enter user/pass" dialog
         doLogin ();
     } else {
@@ -601,11 +612,11 @@ MainWindow::initQML ()
     Q_ASSERT(rv);
     if (!rv) { exit(1); }
     rv = connect (obj, SIGNAL(sigVoicemail(QString)),
-                  this, SLOT(retrieveVoicemail(const QString&)));
+                  oVmail, SLOT(retrieveVoicemail(const QString&)));
     Q_ASSERT(rv);
     if (!rv) { exit(1); }
     rv = connect (obj, SIGNAL(sigVmailPlayback(int)),
-                  this, SLOT(onSigVmailPlayback(int)));
+                  oVmail, SLOT(onSigVmailPlayback(int)));
     Q_ASSERT(rv);
     if (!rv) { exit(1); }
     rv = connect (obj  , SIGNAL(sigMarkAsRead(QString)),
@@ -620,7 +631,7 @@ MainWindow::initQML ()
                   oInbox, SLOT(refreshFullInbox()));
     Q_ASSERT(rv);
     if (!rv) { exit(1); }
-    rv = connect(obj, SIGNAL(sigCloseVmail()), this, SLOT(onSigCloseVmail()));
+    rv = connect(obj, SIGNAL(sigCloseVmail()), oVmail, SLOT(onSigCloseVmail()));
     Q_ASSERT(rv);
     if (!rv) { exit(1); }
     rv = connect(obj, SIGNAL(sigDeleteInboxEntry(const QString &)),
@@ -826,14 +837,9 @@ MainWindow::on_actionE_xit_triggered ()
 {
     this->close ();
 
-    for (QMap<QString,QString>::iterator i  = mapVmail.begin ();
-                                         i != mapVmail.end ();
-                                         i++)
-    {
-        Q_DEBUG(QString("Delete vmail cached at %1").arg (i.value ()));
-        QFile::remove (i.value ());
+    if (NULL != oVmail) {
+        oVmail->onExit ();
     }
-    mapVmail.clear ();
 
 #if MOSQUITTO_CAPABLE
     mqThread.setQuit ();
@@ -870,8 +876,7 @@ MainWindow::getContactsDone (bool bOk)
 void
 MainWindow::initContacts (const QString &contactsPass)
 {
-    oContacts->setUserPass (strUser, contactsPass);
-    oContacts->loginSuccess ();
+    oContacts->login (strUser, contactsPass);
     oContacts->initModel ();
     oContacts->refreshContacts ();
 }//MainWindow::initContacts
@@ -880,7 +885,7 @@ void
 MainWindow::deinitContacts ()
 {
     oContacts->deinitModel ();
-    oContacts->loggedOut ();
+    oContacts->logout ();
 }//MainWindow::deinitContacts
 
 void
@@ -918,8 +923,8 @@ MainWindow::refreshRegisteredNumbers ()
             break;
         }
 
-        rv = connect (token, SIGNAL(completed(AsyncTaskToken*)),
-                      this, SLOT(gotAllRegisteredPhones(AsyncTaskToken*)));
+        rv = connect (token, SIGNAL(completed()),
+                      this, SLOT(gotAllRegisteredPhones()));
         Q_ASSERT(rv);
 
         rv = gvApi.getPhones (token);
@@ -946,8 +951,10 @@ MainWindow::gotRegisteredPhone (const GVRegisteredNumber &info)
 }//MainWindow::gotRegisteredPhone
 
 void
-MainWindow::gotAllRegisteredPhones (AsyncTaskToken *token)
+MainWindow::gotAllRegisteredPhones ()
 {
+    AsyncTaskToken *token = (AsyncTaskToken *) QObject::sender ();
+
     do { // Begin cleanup block (not a loop)
         if (ATTS_SUCCESS != token->status) {
             this->showMsgBox ("Failed to retrieve registered phones");
