@@ -88,6 +88,7 @@ myMessageOutput(QtMsgType type, const char *msg)
     // Send to standard output.
     // I'm not using endl here because endl causes flushes
     cout << strLog.toAscii().constData() << "\n";
+    TOUCHME(strLog);
 
     if ((level <= logLevel) && (NULL != pw)) {
         pw->log(dt, level, strLog);
@@ -115,7 +116,7 @@ static void
 initLogging ()
 {
     OsDependent &osd = Singletons::getRef().getOSD ();
-    QString strLogfile = osd.getAppDirectory ();
+    QString strLogfile = osd.getLogDirectory ();
     strLogfile += QDir::separator ();
     strLogfile += "qgvdial-startup.log";
 
@@ -129,7 +130,7 @@ static void
 initLogRotate()
 {
     OsDependent &osd = Singletons::getRef().getOSD ();
-    QString strLogfile = osd.getAppDirectory ();
+    QString strLogfile = osd.getLogDirectory ();
     strLogfile += QDir::separator ();
     QString oldLogFile = strLogfile;
     oldLogFile += "qgvdial-startup.log";
@@ -185,7 +186,7 @@ main (int argc, char *argv[])
             bQuit = true;
         }
     }
-    if (bQuit == true) {
+    if (bQuit) {
         return (0);
     }
 
@@ -215,6 +216,7 @@ main (int argc, char *argv[])
     app.setApplicationName (APPLICATION_NAME);
 
     int rv = app.exec();
+
     deinitLogging ();
     return (rv);
 }//main
